import java.io.IOException;

/**
 * Template to be used for the first ACO code to tackle TSP 
 * @author Eric Yuan & Sabrina Oliveira
 * @version 0.99
 * Created on Jan 29, 2009
 */
public class FirstACO {

	/**
	 * the number of cities
	 */
	private static int numCities;
	
	/**
	 * the distance matrix
	 */
	private static int[][] distance;
	
	/**
	 * the pheromone matrix
	 */
	private static double[][] pheromone;
	
	/**
	 * the counter for recording the number of iterations
	 */
	private static int iterations;
	
	/**
	 * the counter for recording the number of trials
	 */
	private static int trials;
	
	/**
	 * the maximum number of trials, usually leave it as 1.
	 */
	private static int max_trials = 1;
	
	/**
	 * the maximum number of iterations, for the termination criterion
	 */
	private static int max_iterations;
	
	/**
	 * parameter for pheromone trail evaporation
	 */
	private static double rho;
	
	/**
	 * the number of ants in the colony
	 */
	private static int numAnts;
	
	/**
	 * stores the ant that constructs the best_so_far tour
	 */
	private static Ant bestSoFarAnt;
	
	/**
	 * The main function here. 
	 * @param args
	 */
	public static void main(String[] args) {
		/* variables for main programs are declared above */
		  
		
		init_program(args);   /* initialize all important data */
		
		
		
		for ( trials = 1 ; trials <= max_trials ; trials ++ ) {
		
			startTrial();
		
			while ( !terminationCondition() ) {
		      
				constructSolutions();
		
				evaporatePheromones();
				
				Ant selectedAnt = null; // a loop should be added
		    	depositPheromones(selectedAnt);
		      
		    	iterations++;
			  
			}
		
		    endTrial();
		
		}
		
	}

	private static void endTrial() {
		// TODO Auto-generated method stub
		
	}

	/**
	 * For the selected ants that are allowed to leave pheromone trails on its tour.
	 * @param ant the selected ant to deposit pheromone
	 */
	private static void depositPheromones(Ant ant) {
		// TODO Auto-generated method stub
		
	}

	/**
	 * 
	 *
	 */
	private static void evaporatePheromones() {
		
	}

	private static void constructSolutions() {
		// TODO Auto-generated method stub
		
	}

	private static boolean terminationCondition() {
		return ( iterations >= max_iterations ); 
	}

	private static void startTrial() {
		int i;

	    System.out.println("start trial " + trials);

	    iterations = 1;

	    initializePheromoneMatrix(0.01);
	    
	    printPheromoneMatrix();
		
	}

	/**
	 * read instance data
	 * @param args the input file name
	 */
	private static void init_program(String[] args) {
		System.out.println("start initializing the program......");
		System.out.println("To see how to generate a random (0,1) number: " + Randomizer.ran01());
		String filename = args[0];
		String path = ".";
		InputReader reader = new InputReader(path, filename);
		
		try {
			readProblemSize(filename, reader);
		} catch (IOException e) {
			System.err.println("error while opening file " + filename);
			e.printStackTrace();
		}
		
		readDistanceMatrix(reader);
		
		printDistanceMatrix();

	    //initializePheromoneMatrix(0.001);

	    //printPheromoneMatrix();

	    System.out.println("end init program\n");
		
	}

	/**
	 * initialize the pheromone matrix
	 * @param initValue
	 */
	private static void initializePheromoneMatrix(double initValue) {
		pheromone = new double[numCities][numCities];
		for (int i=0 ; i<pheromone.length ; i++) {
			for (int j=0 ; j<pheromone[i].length ; j++) {
				pheromone[i][j] = initValue;
			}
		}
		
	}

	/**
	 * print distance matrix
	 */
	private static void printDistanceMatrix() {
		StringBuffer s = new StringBuffer("Printing the distance matrix......... \n");
		for (int i=0 ; i<distance.length ; i++) {
			for (int j=0 ; j<distance[i].length ; j++) {
				s.append(distance[i][j]);
				s.append(" ");
			}
			s.append("\n");
		}
		
		System.out.println(s.toString());
	}
	
	/**
	 * print pheromone matrix
	 */
	private static void printPheromoneMatrix() {
		StringBuffer s = new StringBuffer("Printing the pheromone matrix......... \n");
		for (int i=0 ; i<pheromone.length ; i++) {
			for (int j=0 ; j<pheromone[i].length ; j++) {
				s.append(pheromone[i][j]);
				s.append(" ");
			}
			s.append("\n");
		}
		
		System.out.println(s.toString());
	}

	/**
	 * to read the distance matrix from data file.
	 * @param reader for reading input
	 */
	private static void readDistanceMatrix(InputReader reader) {
		distance = new int[numCities][numCities];
		int row = 0;
		
		while (reader.readLine()) {
			//System.out.println(row + " hey " + reader.tokenCount());
			for (int j=0 ; j<numCities ; j++) {
				//System.out.println(row);
				distance[row][j] = reader.nextInt();
			}
			row++;
		}
	}


	/**
	 * read the problem size.
	 * @param filename the input file name
	 * @param reader for input
	 * @throws IOException when the file does not have the correct format.
	 */
	private static void readProblemSize(String filename, InputReader reader) throws IOException {
		if (reader.readLine()) {
			numCities = reader.nextInt();
		} else {
			throw new IOException("The file " + filename + " does not have the correct format.");
		}
	}

}
